package com.hz.pm.api.external.sms;

import cn.hutool.core.lang.TypeReference;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hz.pm.api.external.sms.dto.SmsDTO;
import com.hz.pm.api.external.sms.vo.SmsReceipt;
import com.hz.pm.api.external.sms.vo.SmsReplyResponse;
import com.hz.pm.api.external.sms.vo.SmsSendResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
@RequiredArgsConstructor
public class SmsServiceClient {

    /**
     * 短信服务
     */
    @Value("${sms.client-url:}")
    private String smsUrl;

    public static final String SMS_REPLY = "/sms/reply";
    public static final String SMS_SEND = "/sms/send";
    public static final String SMS_RECEIPT = "/sms/status";
    /**
     * 短信回复
     * @param result
     * @return
     */
    public SmsReplyResponse smsReply(String result) {
        String refreshUrl = smsUrl + SMS_REPLY;
        HashMap<String,Object> map = new HashMap<>();
        map.put("result",result);
        String responseResult = HttpUtil.post(refreshUrl, map);
        JSONObject responseJson = JSON.parseObject(responseResult, JSONObject.class);
        String fileData = responseJson.getString("data");
        return JSONUtil.toBean(fileData, new TypeReference<SmsReplyResponse>() {
        }, false);
    }

    /**
     * 短信发送
     * @return
     */
    public SmsDTO<SmsSendResponse> smsSend(String content, List<String> phones) {
        String phonesSplit = String.join(",", phones);
        String refreshUrl = smsUrl + SMS_SEND;
        Map<String,Object> map = new HashMap<>();
        map.put("content",content);
        map.put("phones",phonesSplit);
        String responseResult = HttpUtil.post(refreshUrl, JSON.toJSONString(map));
        return JSONUtil.toBean(responseResult, new TypeReference<SmsDTO<SmsSendResponse>>() {
        }, false);
    }

    /**
     * 短信回执
     * @param result
     * @return
     */
    public SmsReceipt smsReceipt(String result) {
        String refreshUrl = smsUrl + SMS_RECEIPT;
        HashMap<String,Object> map = new HashMap<>();
        map.put("result",result);
        String responseResult =  HttpUtil.post(refreshUrl, map);
        JSONObject responseJson = JSON.parseObject(responseResult, JSONObject.class);
        String fileData = responseJson.getString("data");
        JSONObject jsonObject = JSON.parseObject(fileData);
        Map<String, Object> dataMap = jsonObject.getInnerMap();
        if(dataMap.get("result") instanceof Boolean){
            return SmsReceipt.builder().error(dataMap.get("error").toString()).build();
        }
        return JSONObject.parseObject(fileData, SmsReceipt.class);
    }

}
