package com.hz.pm.api.projectlib.handle.stage;

import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.enums.StepStatusEnum;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

/**
 * 项目申报处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Component
@RequiredArgsConstructor
public class ProjectDeclareHandle extends AbstractProcessHandle {

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.PROJECT_DECLARED;
    }

    @Override
    public void handle(HandleParamContext ctx) {
        // 因为此时在项目库中的项目，都是已经申报过的项目，所以直接设置环节状态为已完成
        ProcessDetailVO processDetailVO = new ProcessDetailVO();
        processDetailVO.setStepStatus(StepStatusEnum.COMPLETED);
        // 项目库中查出申报完成时间
        Project project = ctx.getProject();
        LocalDateTime createOn = project.getCreateOn();
        processDetailVO.setFinishTime(createOn);
        processDetailVO.setProcessName(getProcessNode().getName());
        ctx.getProcessDetails().add(processDetailVO);
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

}
