package com.hz.pm.api.workbench.manage;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.entity.KeyValDTO;
import com.hz.pm.api.common.statemachine.event.*;
import com.hz.pm.api.common.util.DecimalUtil;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.entity.ProjectGovSystemReplaceInfos;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle.ProcessNode;
import com.hz.pm.api.projectlib.manage.ProjectLibManage;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.GovSystemReplaceTypeEnum;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.service.IProjectGovSystemReplaceInfosService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.hz.pm.api.sys.manage.EarlyWarningManage;
import com.hz.pm.api.sys.model.entity.WflowEarlyWarningRecords;
import com.hz.pm.api.sys.service.IEarlyWarningRecordsService;
import com.hz.pm.api.todocenter.manage.TodoCenterManage;
import com.hz.pm.api.todocenter.model.req.ToBeProcessedReq;
import com.hz.pm.api.todocenter.model.vo.ResToBeProcessedVO;
import com.hz.pm.api.todocenter.model.vo.TodoCenterStatisticsVO;
import com.hz.pm.api.todocenter.model.vo.TodoVO;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.hz.pm.api.workbench.converter.WorkbenchConverter;
import com.hz.pm.api.workbench.model.WorkbenchReq;
import com.hz.pm.api.workbench.model.vo.ProjectProcessStageStatisticsVO;
import com.hz.pm.api.workbench.model.vo.ProjectTotalViewVO;
import com.hz.pm.api.workbench.model.vo.WorkbenchVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.StrPool;
import com.wflow.bean.entity.WflowEarlyWarning;
import com.wflow.service.IEarlyWarningService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.function.ToLongFunction;
import java.util.stream.Collectors;

/**
 * <p>
 * WorkbenchManage
 * </p>
 *
 * @author WendyYang
 * @since 18:58 2024/3/23
 */
@Slf4j
@Component
@AllArgsConstructor
public class WorkbenchManage {

    private final TodoCenterManage todoCenterManage;
    private final IEarlyWarningRecordsService earlyWarningRecordsService;
    private final IEarlyWarningService earlyWarningService;
    private final UserInfoHelper userInfoHelper;
    private final ProjectLibManage projectLibManage;
    private final EarlyWarningManage earlyWarningManage;
    private final IProjectService projectService;
    private final IProjectStatusChangeService projectStatusChangeService;
    private final IPurchaseService purchaseService;
    private final MhUnitCache mhUnitCache;
    private final IProjectGovSystemReplaceInfosService systemReplaceInfosService;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;

    public WorkbenchVO getWorkbenchData(Integer year) {
        WorkbenchVO res = new WorkbenchVO();
        Long userId = LoginUserUtil.getUserId();
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(userId);
        // 待办中心数据
        TodoCenterStatisticsVO todoStat = todoCenterManage.todoCenterStatistics(userId);
        ToBeProcessedReq todoReq = new ToBeProcessedReq();
        todoReq.page(1, 50);
        todoReq.setLoginUserId(userId);
        todoReq.setUserId(userId);
        PageVo<ResToBeProcessedVO> todoPage = todoCenterManage.todoProjectList(todoReq);
        List<TodoVO> todoList = CollUtils.fieldList(todoPage.getRecords(), WorkbenchConverter::convert);
        todoStat.setTodoList(todoList);
        res.setTodoCerter(todoStat);
        // 项目列表
        ProjectListReq projReq = new ProjectListReq();
        projReq.page(1, 20);
        projReq.setProjectYear(year);
        PageVo<ProjectLibListItemVO> projPage = projectLibManage.listWithPermissionWorkBench(projReq, user);
        res.setProjects(new ArrayList<>(projPage.getRecords()));
        return res;
    }

    /**
     * 催办
     *
     * @param recordId \
     * @return \
     */
    public synchronized String urging(Long recordId) {
        WflowEarlyWarningRecords warningRecords = earlyWarningRecordsService.getById(recordId);
        Assert.notNull(warningRecords, "催办失败,id不存在!");
        String projectCode = warningRecords.getProjectCode();
        Project project = projectService.getNoNull(projectCode);
        String batchEmployees = warningRecords.getBatchUserIds();
        Integer ruleType = warningRecords.getRuleType();
        String areaCode = warningRecords.getAreaCode();
        //如果是临期 可以取到 超期的小时数
        String path = warningRecords.getPath();
        LocalDateTime instStart = warningRecords.getInstStart();
        Integer biz = warningRecords.getBiz();
        if (StringUtils.isBlank(batchEmployees)) {
            //如果批次员工号不存在 就取当前
            batchEmployees = warningRecords.getWarningUserId();
        }
        Assert.notEmpty(batchEmployees, "催办失败,员工为空!");
        String[] employees = batchEmployees.split(StrPool.COMMA);
        String noticeContent = warningRecords.getNoticeContent();
        //查出配置的模板
        WflowEarlyWarning config = earlyWarningService.getOne(Wrappers.lambdaQuery(WflowEarlyWarning.class)
                .eq(WflowEarlyWarning::getAreaCode, areaCode)
                .eq(WflowEarlyWarning::getRuleType, ruleType)
                .last(BizConst.LIMIT_1));
        //如果配置能取到 就取模板的
        if (Objects.nonNull(config)) {
            noticeContent = config.getNoticeContent();
        }

        //算出 当前和他的 小时差 临期
        long adventBetween = ChronoUnit.HOURS.between(instStart, LocalDateTime.now());
        String noticeMethod = warningRecords.getNoticeMethod();
        Integer noticeType = warningRecords.getNoticeType();
        for (String employee : employees) {
            earlyWarningManage.urging(noticeMethod, noticeContent,
                    (int) adventBetween, (int) adventBetween,
                    biz, instStart, Long.valueOf(employee),
                    project, ruleType, noticeType, path,
                    batchEmployees, warningRecords.getNodeId());
        }
        return "催办成功了" + employees.length + "个人";
    }

    private void initWorkbenchDefaultParam(WorkbenchReq req) {
        if (req.getUnitId() == null) {
            req.setUnitId(LoginUserUtil.loginUserDetail().getMhUnitId());
        }
        if (req.getProjectYear() == null) {
            req.setProjectYear(LocalDate.now().getYear());
        }
    }

    public ProjectTotalViewVO projectTotalView(WorkbenchReq req) {
        ProjectTotalViewVO retDetail = new ProjectTotalViewVO();
        initWorkbenchDefaultParam(req);
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getReviewAmount, Project::getApprovalGovOwnFinanceAmount, Project::getId)
                .eq(Project::getProjectYear, req.getProjectYear())
                .eq(Project::getNewest, Boolean.TRUE);
        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(req.getUnitId());
        query.in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf));
        List<Project> projects = projectService.list(query);
        if (projects.isEmpty()) {
            return retDetail;
        }
        BigDecimal approvalAmount = DecimalUtil.sum(projects, Project::getApprovalGovOwnFinanceAmount);
        BigDecimal reviewAmount = DecimalUtil.sum(projects, Project::getReviewAmount);
        retDetail.setTotalExecuteAmount(BigDecimal.ZERO);
        retDetail.setTotalApprovalAmount(approvalAmount);
        retDetail.setTotalReviewAmount(reviewAmount);
        Wrapper<ProjectGovSystemReplaceInfos> pgsriQuery = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .in(ProjectGovSystemReplaceInfos::getProjectId,
                        CollUtils.fieldList(projects, Project::getId));
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.list(pgsriQuery);
        Map<String, Long> typeCountMap = systemReplaceInfos.stream()
                .map(w -> StrUtil.split(w.getReplaceType(), ","))
                .flatMap(Collection::stream)
                .collect(Collectors.groupingBy(w -> {
                    if (!NumberUtil.isNumber(w)) {
                        return "其他";
                    } else {
                        return GovSystemReplaceTypeEnum.get(Integer.parseInt(w))
                                .flatMap(t -> Optional.of(t.getVal()))
                                .orElse("其他");
                    }
                }, Collectors.counting()));
        List<KeyValDTO<String, Integer>> systemTypes = typeCountMap.entrySet().stream()
                .map(w -> KeyValDTO.of(w.getKey(), w.getValue().intValue()))
                .collect(Collectors.toList());
        retDetail.setSystemTypes(systemTypes);
        return retDetail;
    }

    public List<ProjectProcessStageStatisticsVO> projectProcessStageStatistics(WorkbenchReq req) {
        initWorkbenchDefaultParam(req);
        List<ProjectProcessStageStatisticsVO> retData = new ArrayList<>();
        List<Long> projectIds = new ArrayList<>();
        Map<String, Long> purchaseEventMap = new HashMap<>();
        ToLongFunction<AbstractStateChangeEvent> getPurchaseCount = event -> purchaseEventMap.getOrDefault(event.name(), 0L);
        long firstInspectedCount = 0;
        for (ProcessNode node : ProcessNode.ALL) {
            long count;
            switch (node) {
                case PROJECT_DECLARED:
                    List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(req.getUnitId());
                    Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                            .select(Project::getId)
                            .eq(Project::getProjectYear, req.getProjectYear())
                            .eq(Project::getNewest, Boolean.TRUE)
                            .in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf));
                    List<Project> projects = projectService.list(query);
                    if (projects.isEmpty()) {
                        return retData;
                    }
                    count = projects.size();
                    projectIds.addAll(CollUtils.fieldList(projects, Project::getId));
                    break;
                case PROJECT_REVIEW:
                    count = 0;
                    break;
                case APPROVAL_AMOUNT:
                    count = projectService.count(Wrappers.lambdaQuery(Project.class)
                            .in(Project::getId, projectIds)
                            .isNotNull(Project::getApprovalAmount));
                    break;
                case PROJECT_APPROVAL:
                    count = projectStatusChangeService.count(Wrappers.lambdaQuery(ProjectStatusChange.class)
                            .in(ProjectStatusChange::getProjectId, projectIds)
                            .eq(ProjectStatusChange::getEvent, ProjectStateChangeEvent.DECLARED_RECORD_PASS));
                    break;
                case PROJECT_PURCHASE:
                    count = purchaseService.count(Wrappers.lambdaQuery(Purchase.class)
                            .in(Purchase::getProjectId, projectIds));
                    if (count > 0) {
                        Wrapper<PurchaseStatusChange> pQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                                .select(PurchaseStatusChange::getEvent)
                                .in(PurchaseStatusChange::getProjectId, projectIds)
                                .in(PurchaseStatusChange::getEvent, AdaptStateChangeEvent.ADAPT_INFO_PASSED,
                                        SelfTestStateChangeEvent.SELF_TEST_PASSED,
                                        TestValidStateChangeEvent.TEST_VALID_INFO_PASSED,
                                        TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES,
                                        XcfhxStateChangeEvent.XCFHX_APPLY_PASSED,
                                        TenderStateChangeEvent.FINALLY_INSPECTED_PASSED);
                        List<PurchaseStatusChange> changes = purchaseStatusChangeService.list(pQuery);
                        changes.forEach(change -> purchaseEventMap.merge(change.getEvent(), 1L, Long::sum));
                    }
                    break;
                case PROJECT_ADAPTION:
                    count = getPurchaseCount.applyAsLong(AdaptStateChangeEvent.ADAPT_INFO_PASSED);
                    break;
                case SYSTEM_SELF_TEST:
                    count = getPurchaseCount.applyAsLong(SelfTestStateChangeEvent.SELF_TEST_PASSED);
                    break;
                case SYSTEM_TEST_VALID:
                    count = getPurchaseCount.applyAsLong(TestValidStateChangeEvent.TEST_VALID_INFO_PASSED);
                    break;
                case FIRST_INSPECTED:
                    count = getPurchaseCount.applyAsLong(TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES);
                    firstInspectedCount = count;
                    break;
                case PILOT_RUNNING:
                    count = firstInspectedCount;
                    break;
                case XCFHX_REVIEW:
                    count = getPurchaseCount.applyAsLong(XcfhxStateChangeEvent.XCFHX_APPLY_PASSED);
                    break;
                case FINAL_INSPECTED:
                    count = getPurchaseCount.applyAsLong(TenderStateChangeEvent.FINALLY_INSPECTED_PASSED);
                    break;
                default:
                    count = 0;
                    break;
            }
            retData.add(ProjectProcessStageStatisticsVO.builder()
                    .stage(node)
                    .stageName(node.getName())
                    .count((int) count)
                    .build());
        }
        return retData;
    }

}
